////////////////////////////////////////////////////////////////////////////////
/** @brief タイマを使って一定間隔でAD変換を行うための設定関数
@date 2012/3/18
*///////////////////////////////////////////////////////////////////////////////
#include <stm32f10x.h>
#include "config_autoADC.h"

////////////////////////////////////////////////////////////////////////////////
/*
使用するペリフェラル
- ADC1
- ADC2
- TIM3
- TIM6
- DMA1-1
- DMA1-2
- DMA1-3
使用するポート
- AD入力ポート
- 汎用出力ポート
*///////////////////////////////////////////////////////////////////////////////
#define RCC_ClockCmd_TIMx_autoADC(x)      RCC_APB2PeriphClockCmd(RCC_APB2Periph_TIM1, x)
#define RCC_ClockCmd_TIMx_sub_autoADC(x)  RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM6, x)
#define RCC_ClockCmd_GPIOx_OUT_autoADC(x) RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOB, x)
#define RCC_ClockCmd_DMAx_autoADC(x)      RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1, x)
#define TIMx_autoADC                      TIM1
#define TIMx_sub_autoADC                  TIM6
#define PCLKx_Frequency_autoADC           PCLK2_Frequency
#define PCLKx_Frequency_sub_autoADC       PCLK1_Frequency
#define GPIOx_OUT_autoADC                 GPIOB
#define DMAx_IT_TCy_endADC_autoADC        DMA1_IT_TC1
#define DMAx_ChannelY_IRQn_endADC_autoADC DMA1_Channel1_IRQn
#define DMAx_ChannelY_endADC_autoADC      DMA1_Channel1
#define DMAx_ChannelY_preADC_OUT_autoADC  DMA1_Channel2
#define DMAx_ChannelY_postADC_OUT_autoADC DMA1_Channel3
#define ADC_ExternalTrigConv_xxx_autoADC  ADC_ExternalTrigConv_T1_CC3
#define GPIO_Pin_x_inA_autoADC (\
	GPIO_Pin_0 | /* ADC123_IN0 */\
	GPIO_Pin_1 | /* ADC123_IN1 */\
	GPIO_Pin_2 | /* ADC123_IN2 */\
	GPIO_Pin_3 | /* ADC123_IN3 */\
	GPIO_Pin_4 | /* ADC12_IN4 */\
	GPIO_Pin_5 | /* ADC12_IN5 */\
	GPIO_Pin_6 | /* ADC12_IN6 */\
	GPIO_Pin_7 | /* ADC12_IN7 */\
0)
#define GPIO_Pin_x_inB_autoADC (\
	GPIO_Pin_0 | /* ADC12_IN8 */\
	GPIO_Pin_1 | /* ADC12_IN9 */\
0)
#define GPIO_Pin_x_inC_autoADC (\
	GPIO_Pin_0 | /* ADC123_IN10 */\
	GPIO_Pin_1 | /* ADC123_IN11 */\
	GPIO_Pin_2 | /* ADC123_IN12 */\
	GPIO_Pin_3 | /* ADC123_IN13 */\
	GPIO_Pin_4 | /* ADC12_IN14 */\
	GPIO_Pin_5 | /* ADC12_IN15 */\
0)
#define GPIO_Pin_x_OUT_autoADC (\
	/*GPIO_Pin_6 | */\
	/*GPIO_Pin_7 | */\
	/*GPIO_Pin_8 | */\
	/*GPIO_Pin_9 | */\
	/*GPIO_Pin_10 |*/\
	/*GPIO_Pin_11 |*/\
0)

struct ADCdata ADC_result[nDMAC_ADC];
uint32_t preADC_GPIOx_OUT_table[nDMAC_ADC];
uint32_t postADC_GPIOx_OUT_table[nDMAC_ADC];
uint16_t subTIM_period;
#define dt_preADC  23.E-6 // [s]
#define dt_postADC  8.E-6  // [s]

////////////////////////////////////////////////////////////////////////////////
/** @brief AD変換対象ポートとサンプリング時間を設定する
@details
SYSCLKが64MHzのときのAD変換器の動作クロック最大値は
10.66...MHzである．このとき，1 クロック = 0.09375 usである．
これを考慮してサンプリングクロック数を設定すること．
@return なし
*///////////////////////////////////////////////////////////////////////////////
void set_ADC_channel()
{
#define T_cpu     ADC_Channel_16 // only ADC1 can convert this channel
#define Vrefint   ADC_Channel_17 // only ADC1 can convert this channel
#define Vbatt     ADC_Channel_8
#define PT_ENC_L  ADC_Channel_1
#define PT_ENC_R  ADC_Channel_9
#define PT_RF     ADC_Channel_12
#define PT_RD     ADC_Channel_10
#define PT_LD     ADC_Channel_11
#define PT_LF     ADC_Channel_13
#define Vref_gyro ADC_Channel_2
#define OUTZ_gyro ADC_Channel_3

// サンプリング時間
#define ST1   ADC_SampleTime_1Cycles5
#define ST7   ADC_SampleTime_7Cycles5
#define ST13  ADC_SampleTime_13Cycles5
#define ST28  ADC_SampleTime_28Cycles5
#define ST41  ADC_SampleTime_41Cycles5
#define ST55  ADC_SampleTime_55Cycles5
#define ST71  ADC_SampleTime_71Cycles5
#define ST239 ADC_SampleTime_239Cycles5

#define SET(order, ch1, ch2, time1, time2) {\
	ADC_RegularChannelConfig(ADC1, ch1, order, time1); \
	ADC_RegularChannelConfig(ADC2, ch2, order, time2); \
}

//             ADC1  ADC2
	SET( 1,Vrefint  , Vbatt    , ST71 , ST71);
	SET( 2,PT_ENC_L , PT_ENC_R , ST71 , ST71);
	SET( 3,PT_ENC_L , PT_ENC_R , ST71 , ST71);
	SET( 4,PT_ENC_L , PT_ENC_R , ST71 , ST71);
	SET( 5,OUTZ_gyro, Vref_gyro, ST71 , ST71);
	SET( 6,PT_LF    , PT_RF    , ST71 , ST71);
	SET( 7,PT_LF    , PT_RF    , ST71 , ST71);
	SET( 8,PT_LF    , PT_RF    , ST71 , ST71);
	SET( 9,T_cpu    , Vbatt    , ST239, ST71);
	SET(10,PT_ENC_L , PT_ENC_R , ST71 , ST71);
	SET(11,PT_ENC_L , PT_ENC_R , ST71 , ST71);
	SET(12,PT_ENC_L , PT_ENC_R , ST71 , ST71);
	SET(13,OUTZ_gyro, Vref_gyro, ST71 , ST71);
	SET(14,PT_LD    , PT_RD    , ST71 , ST71);
	SET(15,PT_LD    , PT_RD    , ST71 , ST71);
	SET(16,PT_LD    , PT_RD    , ST71 , ST71);
}


////////////////////////////////////////////////////////////////////////////////
/** @brief 自動AD変換の設定を行う
@details
タイマを使って一定時間ごとにAD変換を行う．
AD変換後のデータはDMACで自動転送する．
AD変換の前後にDMACを使ってポート出力を切り替えられるようにしているので，\n
LED点灯 --> AD変換 --> LED消灯\n
のプロセスを自動で行うことができる．
また，一連のAD変換終了後に割り込みを発生させる．
@return なし
*///////////////////////////////////////////////////////////////////////////////
void ADC_TIM_DMA_config_for_autoADC()
{
	// クロックを有効にする
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1 | RCC_APB2Periph_ADC2, ENABLE);
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA | RCC_APB2Periph_GPIOB | RCC_APB2Periph_GPIOC, ENABLE);
	RCC_ClockCmd_TIMx_autoADC(ENABLE);
	RCC_ClockCmd_TIMx_sub_autoADC(ENABLE);
	RCC_ClockCmd_GPIOx_OUT_autoADC(ENABLE);
	RCC_ClockCmd_DMAx_autoADC(ENABLE);
	RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE);

	// AD変換入力ポート設定 
	GPIO_InitTypeDef GPIO_InitStructure;
	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_x_inA_autoADC;
	GPIO_Init(GPIOA, &GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_x_inB_autoADC;
	GPIO_Init(GPIOB, &GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_x_inC_autoADC;
	GPIO_Init(GPIOC, &GPIO_InitStructure);

	// DMACで制御する出力ポートの設定
	GPIO_StructInit(&GPIO_InitStructure);
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_x_OUT_autoADC;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOx_OUT_autoADC, &GPIO_InitStructure);

	// 内蔵温度センサ，基準電圧を読めるようにする
	ADC_TempSensorVrefintCmd(ENABLE);

	// ADC1の設定
	ADC_InitTypeDef ADC_InitStructure;
	ADC_StructInit(&ADC_InitStructure);
	ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;
	ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
	ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_xxx_autoADC;
	ADC_InitStructure.ADC_Mode = ADC_Mode_RegSimult;
	ADC_InitStructure.ADC_NbrOfChannel = nADC; // 変換回数
	ADC_InitStructure.ADC_ScanConvMode = DISABLE;
	ADC_Init(ADC1, &ADC_InitStructure);
	// ADC2の設定
	ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;
	ADC_Init(ADC2, &ADC_InitStructure);

	ADC_DiscModeChannelCountConfig(ADC1, 1);
	ADC_DiscModeCmd(ADC1, ENABLE);
	ADC_ExternalTrigConvCmd(ADC1, ENABLE);
	ADC_DiscModeChannelCountConfig(ADC2, 1);
	ADC_DiscModeCmd(ADC2, ENABLE);
	ADC_ExternalTrigConvCmd(ADC2, ENABLE); // ADC1と同時サンプリングENABLE
	
	// ADC1終了後にDMA転送
	ADC_DMACmd(ADC1, ENABLE);

	// AD変換チャネルなどの設定
	set_ADC_channel();

	// タイマのカウンタの設定値を計算する
	uint32_t clock;
	RCC_ClocksTypeDef RCC_Clocks;
	RCC_GetClocksFreq(&RCC_Clocks);
	if(RCC_Clocks.HCLK_Frequency == RCC_Clocks.PCLKx_Frequency_autoADC){
		clock = RCC_Clocks.PCLKx_Frequency_autoADC;
	}
	else{
		clock = RCC_Clocks.PCLKx_Frequency_autoADC * 2;
	}
	uint16_t preDMAC_trig = 2;
	uint16_t ADC_trig = dt_preADC * clock + preDMAC_trig;
	uint16_t postDMAC_trig = dt_postADC * clock + ADC_trig;
	uint16_t ADC_period = dt_int * clock / nDMAC_ADC;
	// エラーチェック
	if(ADC_trig > ADC_period || postDMAC_trig > ADC_period) while(1);

	// AD変換とポート出力制御用タイマの設定
	TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
	TIM_TimeBaseStructInit(&TIM_TimeBaseStructure);
	TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_TimeBaseStructure.TIM_Period = ADC_period - 1;
	TIM_TimeBaseStructure.TIM_Prescaler =  0;
	TIM_TimeBaseInit(TIMx_autoADC, &TIM_TimeBaseStructure);
	// AD変換タイミング (CC3)
	TIM_OCInitTypeDef TIM_OCInitStructure;
	TIM_OCStructInit(&TIM_OCInitStructure);
	TIM_OCInitStructure.TIM_OCMode = TIM_OCMode_PWM2;
	TIM_OCInitStructure.TIM_OCPolarity = TIM_OCPolarity_High;
	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
	TIM_OCInitStructure.TIM_Pulse = ADC_trig - 1;
	TIM_OC3Init(TIMx_autoADC, &TIM_OCInitStructure);
	TIM_CtrlPWMOutputs(TIMx_autoADC, ENABLE);
	// AD変換前DMA転送タイミング (CC1)
	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Disable;
	TIM_OCInitStructure.TIM_Pulse = preDMAC_trig - 1;
	TIM_OC1Init(TIMx_autoADC, &TIM_OCInitStructure);
	// AD変換後DMA転送タイミング (CC2)
	TIM_OCInitStructure.TIM_Pulse = postDMAC_trig - 1;
	TIM_OC2Init(TIMx_autoADC, &TIM_OCInitStructure);
	// DMAC起動許可
	TIM_DMACmd(TIMx_autoADC, TIM_DMA_CC1, ENABLE);
	TIM_DMACmd(TIMx_autoADC, TIM_DMA_CC2, ENABLE);

	// regular group変換後データ転送用DMACの設定
	DMA_InitTypeDef DMA_InitStructure;
	DMA_StructInit(&DMA_InitStructure);
	DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)(&(ADC1->DR));
	DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)ADC_result;
	DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;
	DMA_InitStructure.DMA_BufferSize = nDMAC_ADC; // 転送回数
	DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
	DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
	DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Word;
	DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Word;
	DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
	DMA_InitStructure.DMA_Priority = DMA_Priority_Medium;
	DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
	DMA_Init(DMAx_ChannelY_endADC_autoADC, &DMA_InitStructure);
	// 指定回数の転送終了後に割り込みを行う
	NVIC_InitTypeDef NVIC_InitStructure;
	NVIC_InitStructure.NVIC_IRQChannel = DMAx_ChannelY_IRQn_endADC_autoADC;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
	DMA_ITConfig(DMAx_ChannelY_endADC_autoADC, DMA_IT_TC, ENABLE);

	// AD変換前にポート出力を切り替えるためのDMACの設定
	DMA_StructInit(&DMA_InitStructure);
	DMA_InitStructure.DMA_PeripheralBaseAddr = (uint32_t)(&(GPIOx_OUT_autoADC->BSRR));
	DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)preADC_GPIOx_OUT_table;
	DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralDST;
	DMA_InitStructure.DMA_BufferSize = nDMAC_ADC; // 転送回数
	DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
	DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
	DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_Word;
	DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_Word;
	DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
	DMA_InitStructure.DMA_Priority = DMA_Priority_VeryHigh;
	DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
	DMA_Init(DMAx_ChannelY_preADC_OUT_autoADC, &DMA_InitStructure);
	// AD変換後にポート出力を切り替えるためのDMACの設定
	DMA_InitStructure.DMA_MemoryBaseAddr = (uint32_t)postADC_GPIOx_OUT_table;
	DMA_Init(DMAx_ChannelY_postADC_OUT_autoADC, &DMA_InitStructure);

	// 周期割り込み補助タイマ
	if(RCC_Clocks.HCLK_Frequency == RCC_Clocks.PCLKx_Frequency_sub_autoADC){
		clock = RCC_Clocks.PCLKx_Frequency_sub_autoADC;
	}
	else{
		clock = RCC_Clocks.PCLKx_Frequency_sub_autoADC * 2;
	}
	uint32_t period = dt_int * clock;
	uint16_t subTIM_prescalar = 1;
	while(1){
		subTIM_period = period / subTIM_prescalar;
		if(subTIM_period < 0x800) break;
		subTIM_prescalar <<= 1;
	}
	TIM_TimeBaseStructInit(&TIM_TimeBaseStructure);
	TIM_TimeBaseStructure.TIM_ClockDivision = TIM_CKD_DIV1;
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_TimeBaseStructure.TIM_Period = 0xFFFF;
	TIM_TimeBaseStructure.TIM_Prescaler = subTIM_prescalar - 1;
	TIM_TimeBaseInit(TIMx_sub_autoADC, &TIM_TimeBaseStructure);

	// ADCの有効化とキャリブレーション
	ADC_Cmd(ADC1, ENABLE);
	ADC_ResetCalibration(ADC1);
	while(ADC_GetResetCalibrationStatus(ADC1));
	ADC_StartCalibration(ADC1);
	while(ADC_GetCalibrationStatus(ADC1));
	ADC_Cmd(ADC2, ENABLE);
	ADC_ResetCalibration(ADC2);
	while(ADC_GetResetCalibrationStatus(ADC2));
	ADC_StartCalibration(ADC2);
	while(ADC_GetCalibrationStatus(ADC2));

	// Enable DMA
	DMA_Cmd(DMAx_ChannelY_endADC_autoADC, ENABLE);
	DMA_Cmd(DMAx_ChannelY_preADC_OUT_autoADC, ENABLE);
	DMA_Cmd(DMAx_ChannelY_postADC_OUT_autoADC, ENABLE);

	// タイマ動作開始
	TIM_Cmd(TIMx_sub_autoADC, ENABLE);
	TIM_Cmd(TIMx_autoADC, ENABLE);
}


volatile float abs_time = 0; // [s]
volatile uint64_t time_count = 0;
int duty_int_routine;
int duty_int_routine_max = 0;
int subTIM_value;
////////////////////////////////////////////////////////////////////////////////
/** @brief 割り込みルーチンのはじめに実行されるべき関数
@details
- 割り込みフラグのクリア
- 時間の更新
を行う
@return
なし
*///////////////////////////////////////////////////////////////////////////////
void int_pre_process()
{
	// 割り込みフラグのクリア
	DMA_ClearITPendingBit(DMAx_IT_TCy_endADC_autoADC);

	// 時間の更新
	static uint16_t value_old = 0;
	subTIM_value = TIM_GetCounter(TIMx_sub_autoADC);
	time_count += (uint16_t)(subTIM_value - value_old);
	value_old = subTIM_value;
	abs_time = time_count / subTIM_period * dt_int;
}


////////////////////////////////////////////////////////////////////////////////
/** @brief 割り込み周期に占める呼び出し位置までの処理時間の割合
@details
割り込み内の処理にかかった時間を計算する
@return
なし
*///////////////////////////////////////////////////////////////////////////////
int duty_here()
{
	return (uint16_t)(TIM_GetCounter(TIMx_sub_autoADC) - subTIM_value)
	       * 1000 / subTIM_period;
}


////////////////////////////////////////////////////////////////////////////////
/** @brief 割り込みルーチンの終わりに実行されるべき関数
@details
割り込み内の処理にかかった時間を計算する
@return
なし
*///////////////////////////////////////////////////////////////////////////////
void int_post_process()
{
	// 処理の重さの計算
	duty_int_routine = duty_here();
	if(duty_int_routine > duty_int_routine_max) duty_int_routine_max = duty_int_routine;
}


////////////////////////////////////////////////////////////////////////////////
/** @brief 周期割り込みを禁止する
@details
FLASHへの書き込み中に割り込みがかかると長時間保留される可能性がある．
このときに生じうる不具合を防ぐためにあらかじめ割り込みを禁止するために使用する．
@return
なし
*///////////////////////////////////////////////////////////////////////////////
void stop_periodic_interrupt()
{
	DMA_ITConfig(DMAx_ChannelY_endADC_autoADC, DMA_IT_TC, DISABLE);
}


////////////////////////////////////////////////////////////////////////////////
/** @brief stop_periodic_interrupt()で禁止した割り込みを再度許可する
@return
なし
*///////////////////////////////////////////////////////////////////////////////
void restart_periodic_interrupt()
{
	DMA_ClearITPendingBit(DMAx_IT_TCy_endADC_autoADC); // 割り込みフラグのクリア
	DMA_ITConfig(DMAx_ChannelY_endADC_autoADC, DMA_IT_TC, ENABLE);
}

